/*
 *
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Florian Spreckelsen <f.spreckelsen@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License along with this program. If not, see
 * <https://www.gnu.org/licenses/>.
 *
 */
#include <string>

#include <jlcxx/jlcxx.hpp>
#include <jlcxx/array.hpp>
#include <jlcxx/const_array.hpp>
#include <jlcxx/functions.hpp>
#include <jlcxx/module.hpp>
#include <jlcxx/smart_pointers.hpp>
#include <jlcxx/stl.hpp>
#include <jlcxx/tuple.hpp>
#include <jlcxx/type_conversion.hpp>

#include "caosdb/authentication.h"
#include "caosdb/connection.h"
#include "caosdb/constants.h"
#include "caosdb/exceptions.h"

// namespace jlcxx
// {
//   template<>
//   struct SuperType<caosdb::authentication::PlainPasswordAuthenticator>
//   {
//     typedef caosdb::authentication::Authenticator type;
//   };
// }

JLCXX_MODULE define_julia_module(jlcxx::Module& mod)
{
  // Version info of libcaosdb against which this is built.
  mod.method("get_libcaosdb_version_major", []() {return caosdb::LIBCAOSDB_VERSION_MAJOR;});
  mod.method("get_libcaosdb_version_minor", []() {return caosdb::LIBCAOSDB_VERSION_MINOR;});
  mod.method("get_libcaosdb_version_patch", []() {return caosdb::LIBCAOSDB_VERSION_PATCH;});

  // exceptions inherit from abstract Julia exception types:
  mod.add_type<caosdb::exceptions::AuthenticationError>("AuthenticationError",
							jlcxx::julia_type("AbstractAuthenticationError"))
    .constructor<const std::string&>()
    .method("what", &caosdb::exceptions::AuthenticationError::what);
  mod.add_type<caosdb::exceptions::ConnectionError>("ConnectionError",
						    jlcxx::julia_type("AbstractConnectionError"))
    .constructor<const std::string&>()
    .method("what", &caosdb::exceptions::ConnectionError::what);

  // authenticators
  mod.add_type<caosdb::authentication::Authenticator>("CaosDBAuthenticator");
  // Currently not working, see https://github.com/JuliaInterop/CxxWrap.jl/issues/294
  // mod.add_type<caosdb::authentication::PlainPasswordAuthenticator>("PlainPasswordAuthenticator",
  // 								   jlcxx::julia_base_type<caosdb::authentication::Authenticator>())
  //   .constructor<const std::string&, const std::string&>()
  //   .method("getCallCredentials", &caosdb::authentication::PlainPasswordAuthenticator::getCallCredentials);

  mod.add_type<caosdb::connection::CACertificateProvider>("CACertificateProvider");
  // TODO Add children once the above issue is resolved
  
  mod.add_type<caosdb::connection::CaosDBConnectionConfig>("CaosDBConnectionConfig");
  mod.add_type<caosdb::connection::InsecureCaosDBConnectionConfig>("InsecureCaosDBConnectionConfig");
}
