# Useful for development, TODO remove for production
__precompile__(false)

module CaosDB

using CxxWrap

"Generic super type of CaosDB exceptions"
abstract type AbstractCaosDBException <: Exception end

"An error occurred while trying to authenticate"
abstract type AbstractAuthenticationError <: AbstractCaosDBException end

"Connection couldn't be established"
abstract type AbstractConnectionError <: AbstractCaosDBException end

# Adapt this path if you haven't compiled jlcaosdb to `caosdb-julialib/build`
@wrapmodule(joinpath(@__DIR__, "../jlcaosdb/build/lib/libjlcaosdb"))

function __init__()
    @initcxx
end

"""
    get_libcaosdb_version()

Returns the version string of libcaosdb with which this package has
been built.
"""
function get_libcaosdb_version()
    major = get_libcaosdb_version_major()
    minor = get_libcaosdb_version_minor()
    patch = get_libcaosdb_version_patch()

    return "$major.$minor.$patch"
end

# Override the exception printing
Base.showerror(io::IO, e::AuthenticationError) = print(io, what(e))
Base.showerror(io::IO, e::ConnectionError) = print(io, what(e))

end # module
